import { deepClone, hasOwnProperty, warn } from '@/utils/util'

export function getFormData(formData, items, options) {
  return new FormDataFormat(options).getFormData(formData, items)
}

export default class FormDataFormat {
  constructor(options) {
    this.options = options
  }
  /**
   * 将表单绑定数据按照dataFormat格式化转换成待提交数据
   * @param {Object} formData 表单绑定数据
   * @param {Array} items 表单子项数组
   * @returns 待提交格式化后的表单数据
   */
  getFormSubmitData(formData, items) {
    const data = deepClone(formData)
    items.forEach(i => {
      // 判断是否需要提交， 如果不需要则返回
      if (i.notSubmit) {
        delete data[i.key]
        return
      }
      // 判断是否日期时间类控件
      if (this.setRangeData(i, formData, data)) {
        delete data[i.key]
        return
      }
      // 通过是否有数据格式化函数，如果有则直接调用，自己不做处理
      if (typeof i.dataFormat === 'function') {
        const result = i.dataFormat(i, formData, data)
        if (result !== undefined) {
          data[i.key] = result
        }
        return
      }
      // 通过数据格式化获取提交数据， 如果没设置则获取原始数据
      let value = this.getDataByDataFormat(i, formData)
      if (value === null) {
        value = data[i.key]
      }
      // 判断是否有包装对象， 如果有则添加到包装对象中，
      if (i.wrapperObject) {
        if (!data[i.wrapperObject]) {
          data[i.wrapperObject] = {}
        }
        const field = i.field || i.key
        data[i.wrapperObject][field] = value
        delete data[i.key]
        return
      }
      // 判断是否有设置字段名， 如果有则添加到字段名中，并删除源属性
      if (i.field) {
        data[i.field] = value
        delete data[i.key]
        return
      }
      data[i.key] = value
    })
    return data
  }
  /**
   * 设置日期时间范围的数据
   */
  setRangeData(item, formData, data) {
    if (
      ['time', 'date', 'datetime'].includes(item.type) &&
      item?.props?.isRange
    ) {
      const value = formData[item.key]
      if (Array.isArray(value)) {
        if (item.rangeKeys) {
          if (Array.isArray(item.rangeKeys)) {
            item.rangeKeys.forEach((i, index) => {
              data[i] = value[index]
            })
            return true
          } else {
            warn(`[ldp-form dataFormat warn] ${item.key} rangeKeys expect array`)
          }
        }
      }
    }
  }
  setRangeFormData(item, formData, data) {
    if (
      ['time', 'date', 'datetime'].includes(item.type) &&
      item?.props?.isRange
    ) {
      // 判断是否设置了 范围key， 如果有设置则需要按照对应顺序获取值，存放当数组中
      if (item.rangeKeys) {
        const list = []
        item.rangeKeys.forEach((i) => {
          list.push(formData[i] || '')
          delete data[formData[i]]
        })
        data[item.key] = list
        return true
      }
    }
  }
  /**
   * 通过数据格式化获取实际提交数据
   */
  getDataByDataFormat(item, formData) {
    let dataFormat = item.dataFormat
    if (!dataFormat) {
      if (this.options?.dataFormat) {
        // 判断是否同类型，如果同类型则自动转换格式
        if (hasOwnProperty(this.options.dataFormat, item.type)) {
          dataFormat = this.options.dataFormat[item.type]
        }
      }
    }
    if (!dataFormat) return null
    const value = formData[item.key]
    if (typeof dataFormat === 'function') {
      return dataFormat(value, formData, item)
    }
    if (value === null || value === undefined) return null
    if (typeof dataFormat === 'string') {
      // 判断是否是分割， 默认分割为 ，
      if (dataFormat === 'split' && Array.isArray(value)) {
        return this.formatSplit(value)
      } else if (['number', Number].includes(dataFormat)) {
        if (typeof value === 'boolean') return value ? 1 : 0
        if (typeof value === 'object') {
          warn(`[ldp-form dataFormat warn] ${item.key} type error`)
          return value
        }
        return Number(value)
      } else if (dataFormat === 'json') {
        return JSON.stringify(value)
      } else if (['object', Object].includes(dataFormat)) {
        // 保存为对象类型，先判断当前类型是否为对象， 如果是则直接保存，如果不是则自动封装成对象， 默认key为id
        if (typeof value === 'object') return value
        return {
          id: value
        }
      } else if (['boolean', Boolean].includes(dataFormat)) {
        if (value !== 0) {
          return true
        }
        return false
      } else if (value && typeof value === 'object' && dataFormat in value) {
        return value[dataFormat]
      }
    } else if (typeof dataFormat === 'object') {
      if (dataFormat.split) {
        return this.formatSplit(value, dataFormat.split)
      } else if (dataFormat.filterFields) {
        if (Array.isArray(dataFormat.filterFields)) {
          const data = {}
          dataFormat.filterFields.forEach(i => {
            data[i] = value[i]
          })
          return data
        }
        warn(
          `[ldp-form dataFormat warn] ${item.key} filterFields is not array`
        )
        return value
      } else if (dataFormat.valueField) {
        return value[dataFormat.valueField]
      }
    }
    return null
  }
  /**
   * 通过数据格式化获取组件绑定数据
   */
  getFormDataByDataFormat(item, formData) {
    let dataFormat = item.dataFormat
    if (!dataFormat) {
      if (this.options?.dataFormat) {
        // 判断是否同类型，如果同类型则自动转换格式
        if (hasOwnProperty(this.options.dataFormat, item.type)) {
          dataFormat = this.options.dataFormat[item.type]
        }
      }
    }
    if (!dataFormat) return null
    const value = formData[item.key]

    if (value === null || value === undefined) return value
    if (typeof dataFormat === 'string') {
      // 判断是否是分割， 默认分割为 ，
      if (dataFormat === 'split') {
        if (item.type === 'select') {
          if (item.multiple || item?.props?.multiple) {
            return value.split(',')
          }
          return value
        }
        return value.split(',')
      } else if (['number', Number].includes(dataFormat)) {
        if (typeof value === 'number') return Boolean(value)
        if (typeof value === 'object') {
          warn(`[ldp-form dataFormat waring] ${item.key} type error`)
          return value
        }
        return Boolean(value)
      } else if (dataFormat === 'json') {
        if (typeof value === 'string') return JSON.parse(value)
        return value
      } else if (['object', Object].includes(dataFormat)) {
        // 保存为对象类型，先判断当前类型是否为对象， 如果是则直接保存，如果不是则自动封装成对象， 默认key为id
        return value
      }
    } else if (typeof dataFormat === 'object') {
      if (dataFormat.split) {
        return this.formatSplit(value, dataFormat.split)
      } else if (dataFormat.filterFields) {
        return value
      }
    }
    return null
  }
  /**
   *  当数组数据转换成字符串， 默认,分割
   * @param {*} value 当前表单子项的值
   * @param {*} formatStr 格式化分割的字符串，默认,分割
   * @returns
   */
  formatSplit(value, formatStr = ',') {
    // 判断是否是对象，如果成员是对象，则默认从id获取。
    const list = value.map(v => {
      if (typeof v === 'object' && v !== null) {
        if (v.id) return v.id
      }
      return v
    })
    return list.join(formatStr)
  }
  /**
   *
   * 从字符串转换成数组
   */
  formatSplit2Ary(value, formatStr = ',') {
    if (typeof value === 'string') {
      return value.split(formatStr)
    }
    return value
  }
  /**
   * 一般用于通过接口获取到的数据，然后还原成绑定表单需要的数据。
   * @param {*} formData
   * @param {*} items
   * @returns
   */
  getFormData(formData, items) {
    const data = formData
    items.forEach(async i => {
      // 判断是否日期时间范围内控件，如果是则需要设置成数组。
      if (this.setRangeFormData(i, formData, data)) {
        return
      }
      if (i.relation) {
        this.getRelationData(formData, i)
        return
      }
      // 通过是否有数据格式化函数，如果有则直接调用，自己不做处理
      if (typeof i.formDataFormat === 'function') {
        try {
          const res = await i.formDataFormat(data[i.key], i, formData)
          if (res !== undefined) data[i.key] = res
        } catch (error) {
          warn('[ldp-form warn] formDataFormat error')
        }
        return
      }
      // 通过数据格式化获取提交数据， 如果没设置则获取原始数据
      let value = this.getFormDataByDataFormat(i, formData)
      if (value === null) {
        value = data[i.key]
      }
      // 判断是否有包装对象， 按照包装对象还原，最后删除包装对象。
      if (i.wrapperObject) {
        if (data[i.wrapperObject]) {
          const tmp = data[i.wrapperObject]
          const field = i.field || i.key
          data[i.key] = tmp[field]
          delete tmp[field]
          if (Object.keys(tmp).length === 0) {
            delete data[i.wrapperObject]
          }
        }
        return
      }
      // 判断是否有设置字段名， 如果有则添加到字段名中，并删除源属性
      if (i.field) {
        data[i.key] = data[i.field]
        delete data[i.field]
        return
      }
      data[i.key] = value || undefined
    })
    return data
  }
  /**
   * 获取关联数据，并删除关联key
   */
  getRelationData(formData, item) {
    const relation = item.relation
    let relationKey = relation
    let value = formData[relation]
    if (typeof relation === 'object' && relation.key) {
      if (!(relation.key in formData)) {
        return
      }
      relationKey = relation.key
      value = formData[relation.key]
      if (typeof value === 'object' && value) {
        formData[item.key] = value[relation.value]
      } else {
        formData[item.key] = value
      }
    } else {
      formData[item.key] = value
    }
    delete formData[relationKey]
  }
}
