
export function hasOwnProperty(obj, key) {
  return Object.prototype.hasOwnProperty.call(obj, key)
}

export function deepClone(obj) {
  const objClone = Array.isArray(obj) ? [] : {}
  if (obj && typeof obj === 'object') {
    for (const key in obj) {
      if (obj[key] && typeof obj[key] === 'object') {
        objClone[key] = deepClone(obj[key])
      } else {
        objClone[key] = obj[key]
      }
    }
  }
  return objClone
}

// 帕斯卡 首字母大写
export function toPascalCase(text) {
  return text.charAt(0).toUpperCase() + text.substr(1)
}

// 骆驼
export function toCamelCase(text) {
  return text.replace(/^([A-Z])|[\s-_]+(\w)/g, function(match, p1, p2, offset) {
    if (p2) return p2.toUpperCase()
    return p1.toLowerCase()
  })
}

export function debounce(fun, delay) {
  let timer = null
  return function() {
    const that = this
    const _args = arguments
    clearTimeout(timer)
    timer = setTimeout(function() {
      fun.apply(that, _args)
    }, delay)
  }
}

/**
 * @param {string} url
 * @returns {Object}
 */
export function param2Obj(url) {
  const search = url.split('?')[1]
  if (!search) {
    return {}
  }
  return JSON.parse(
    '{"' +
      decodeURIComponent(search)
        .replace(/"/g, '\\"')
        .replace(/&/g, '","')
        .replace(/=/g, '":"')
        .replace(/\+/g, ' ') +
      '"}'
  )
}

/**
 * @desc 格式化时间
 * @param {String} date 时间
 * @param {String} fmt 格式化配置字符串
 * @return {String} 返回格式化后的字符串时间
 */
export function formatDate2Str(date, fmt) {
  var currentDate = new Date(date)
  var o = {
    'M+': currentDate.getMonth() + 1, // 月份
    'd+': currentDate.getDate(), // 日
    'h+': currentDate.getHours(), // 小时
    'm+': currentDate.getMinutes(), // 分
    's+': currentDate.getSeconds(), // 秒
    'q+': Math.floor((currentDate.getMonth() + 3) / 3), // 季度
    'S': currentDate.getMilliseconds() // 毫秒
  }
  if (/(y+)/.test(fmt)) fmt = fmt.replace(RegExp.$1, (currentDate.getFullYear() + '').substr(4 - RegExp.$1.length))
  for (const k in o) {
    if (new RegExp('(' + k + ')').test(fmt)) fmt = fmt.replace(RegExp.$1, (RegExp.$1.length === 1) ? (o[k]) : (('00' + o[k]).substr(('' + o[k]).length)))
  }
  return fmt
}

/**
 * @description 生成uuid 默认是30为uuid 设置长度则是随机长度字符串
 * @param {Number} len 生成字符串的长度 如果不填默认是uuid
 * @param {Number} radix 支持的进制 2 10 16进制 默认16
 */
export function uuid(len = 30, radix = 16) {
  const chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'.split('')
  const uuid = []
  let i = 0
  if (len) {
    const charLen = radix || chars.length
    for (i = 0; i < len; i++) uuid[i] = (chars[0 | Math.random() * charLen])
    return uuid.join('')
  }
  return uuid.join('')
}

/**
 * 输出警告
 */
export function warn(msg) {
  if (process.env.NODE_ENV === 'development') {
    console.warn(msg)
  }
}

/**
 * 判断是否为空
 * @param {*} value
 * @returns
 */
export function isEmpty(value) {
  if (value === '' || value === undefined || value === null) {
    return true
  }
  return false
}

/**
 * 判断两个对象是否数据相等
 * @param {Object | Array} obj1
 * @param {Object | Array} obj2
 * @param {Array} filterProtoerys 过滤的属性 不判断此属性
 */
export function isEqual(obj1, obj2, filterProtoerys) {
  // undefined 也会判断为相等
  if (!obj1) return obj1 === obj2
  if (!obj2) return obj1 === obj2
  if (!(obj1 instanceof Object) || !(obj2 instanceof Object)) { /*  判断不是对象  */
    return obj1 === obj2
  }
  if (!filterProtoerys) {
    // 判断对象key的个数是否相等 不相等直接返回false
    if (Object.keys(obj1).length !== Object.keys(obj2).length) {
      return false
    }
  }
  for (var key in obj1) {
    // 判断是否有过滤属性 或者过滤属性数组
    if (filterProtoerys) {
      if (Array.isArray(filterProtoerys)) {
        if (filterProtoerys.includes(key)) {
          continue
        }
      } else if (filterProtoerys === key) {
        continue
      }
    }
    // 判断是否是对象 如果是则递归
    if (obj1[key] instanceof Object && obj2[key] instanceof Object) {
      if (!isEqual(obj1[key], obj2[key], filterProtoerys)) {
        return false
      }
    } else if (obj1[key] !== obj2[key]) {
      return false
    }
  }
  return true
}

/**
 * 通过字段名称获取数据
 * @param {*} data 数据
 * @param {*} field 字段名，支持.
 */
export function getFieldData(data, field) {
  const ary = field.split('.')
  let obj = data
  for (let i = 0; i < ary.length; i++) {
    // 判断指定的内容是否，存在， 判断是否为对象。如果是对象则继续下一级
    if (obj[ary[i]]) {
      obj = obj[ary[i]]
    }
  }
  return obj
}

// 解析转换url
export function parseUrl(url, obj = {}) {
  Object.keys(obj).forEach(key => {
    const reg = [new RegExp(`{${key}}`, 'g'), new RegExp(`:${key}`, 'g')]
    reg.find(i => {
      if (i.test(url)) {
        url = url.replace(i, obj[key])
        return i
      }
    })
  })
  return url
}

/**
 * 判断两个对象里面指定的属性名的值是否相等
 * @param {Object | Array} obj1
 * @param {Object | Array} obj2
 * @param {Array} protoerys 需要对比的属性名
 */
export function isEqualProtoerys(obj1, obj2, protoerys = []) {
  // undefined 也会判断为相等
  if (!obj1) return obj1 === obj2
  if (!obj2) return obj1 === obj2
  if (!(obj1 instanceof Object) || !(obj2 instanceof Object)) { /*  判断不是对象  */
    return obj1 === obj2
  }
  for (const index in protoerys) {
    const key = protoerys[index]
    if (obj1[key] === obj2[key]) {
      continue
    } else if (typeof obj1[key] === typeof obj2[key]) {
      return false
    } else if (obj1[key] && obj2[key]) {
      // 判断值是否一样 不一样则直接返回
      if (obj1[key].toString() !== obj2[key].toString()) {
        return false
      }
    } else if (!obj1[key] && !obj2[key]) {
      continue
    } else {
      return false
    }
  }
  return true
}

/**
 * 格式化文件大小, 输出成带单位的字符串
 * @param {Number} size 文件大小
 * @param {Number} [pointLength=2] 精确到的小数点数。
 * @param {Array} [units=[ 'B', 'K', 'M', 'G', 'TB' ]] 单位数组。从字节，到千字节，一直往上指定。
 *    如果单位数组里面只指定了到了K(千字节)，同时文件大小大于M, 此方法的输出将还是显示成多少K.
 */
export function formatSize(size, pointLength, units) {
  var unit
  if (size === '' || size === null || size === undefined) {
    return ''
  }
  if (isNaN(size)) {
    return ''
  }
  units = units || ['B', 'K', 'M', 'G', 'TB']
  while ((unit = units.shift()) && size > 1024) {
    size = size / 1024
  }
  return (unit === 'B' ? size : size.toFixed(pointLength === undefined ? 2 : pointLength)) + unit
}

